require "spec_helper"

describe Permalinkable do
  context "Non-changable permalink" do
    before(:each) do
      User.delete_all
      Entry.delete_all
    end

    it 'injects permalink_options as class attribute to the model' do
      expect(User.permalink_options).to include({
            :permalinkable_attribute => :user_name,
               :permalink_field_name => :permalink,
                             :length => 100,
                       :allow_change => false
      })
    end

    it 'prevents permalink_options from being modified' do
      expect{ User.permalink_options.merge!(test: 1) }.to raise_error(RuntimeError)
    end

    it 'adds presence and non-empty validation on permalinkable_attribute' do
      expect(User.create.errors.messages).to include({
        user_name: ["can't be blank", "is too short (minimum is 0 characters)"]
      })

      expect(Entry.create.errors.messages).to include({
        entry_name: ["can't be blank", "is too short (minimum is 0 characters)"]
      })
    end

    it 'generates permalink for new model' do
      u = User.create(user_name: 'test user')
      expect(u.permalink).not_to be_empty()

      e = Entry.create(entry_name: 'test entry')
      expect(e.entry_identity).not_to be_empty()
    end

    it 'generates decrypteable permalink' do
      u = User.create(user_name: 'test user')
      dec = User.permalink_cipher
      dec.decrypt
      expect(
        dec.update(Base64.urlsafe_decode64(u.permalink.gsub("-test-user", ''))).strip
      ).to eq(u.id.to_s)
    end

    it 'prevents permalink to be manully modified' do
      u = User.create(user_name: 'test user')
      permalink = u.permalink
      u.update_attributes(permalink: '123123') && u.reload
      expect(u.permalink).to eq(permalink)

      e = Entry.create(entry_name: 'test entry')
      permalink = e.entry_identity
      e.update_attributes(entry_identity: '123123') && e.reload
      expect(e.entry_identity).to eq(permalink)
    end

    it 'makes to_param to return permalink if available' do
      u = User.create(user_name: 'test user')
      expect(u.permalink).to eq(u.to_param)

      e = Entry.create(entry_name: 'test entry')
      expect(e.entry_identity).to eq(e.to_param)
    end
  end

  context "Changable permalink" do
    before(:each) do
      Entry.delete_all
    end

    it 'changes permalink once permalinkable_attribute changes' do
      e = Entry.create(entry_name: 'test entry')
      old_permalink = e.entry_identity
      e.update_attributes(entry_name: 'changed entry')
      new_permalink = e.entry_identity
      expect(old_permalink).not_to eq(new_permalink)
      expect(old_permalink.first(4)).to eq(new_permalink.first(4))
    end
  end
end